import db from "../../db"
import { Context } from "../../types"
import llm from "../../utils/llm"
import { extractSheetIds, validateLink } from "../../utils/string"

/**
 * 爬取网页内容
 * @param {Context} ctx - 上下文对象
 * @param {string} link - 网页链接
 * @returns {Promise<any>} - 返回爬取结果
 * @throws {Error} - 当爬取失败时抛出错误
 */
const crawlWebPage = async (ctx: Context, link: string): Promise<any> => {
  const { attachService } = ctx
  const crawRes = await attachService.crawlWeb(link)
  if (!crawRes || crawRes?.code) throw new Error("网页抓取失败")
  return crawRes
}

/**
 * 生成网页简报
 * @param {Context} ctx - 上下文对象
 * @param {string} userDescription - 用户描述
 * @param {any} content - 网页内容
 * @returns {Promise<string>} - 返回简报内容
 * @throws {Error} - 当生成简报失败时抛出错误
 */
const generateSummary = async (
  ctx: Context,
  userDescription: string,
  content: any
): Promise<string> => {
  const { requestId } = ctx
  const llmRes = (await llm.invoke(
    "summaryWeb",
    {
      description: userDescription,
      content: content,
    },
    requestId,
    1
  )) as string
  if (!llmRes) throw new Error("模型总结失败")
  return llmRes
}

/**
 * 插入到表格
 * @param {Context} ctx - 上下文对象
 * @param {string} link - 网页链接
 * @param {string} userDescription - 用户描述
 * @param {string} llmRes - 简报内容
 * @returns {Promise<string>} - 返回表格链接
 */
const insert2Sheet = async (
  ctx: Context,
  link: string,
  userDescription: string,
  llmRes: string
) => {
  const {
    larkBody: { userId },
    logger,
    larkService,
  } = ctx
  try {
    const chat = await db.chat.getAndCreate(ctx)
    if (!chat?.webSummarySheetLink) {
      logger.info("No webSummarySheetLink found, skip insert2Sheet")
      return ""
    }
    let sheetToken = ""
    let range = ""
    if (chat.webSummarySheetLink.includes("wiki")) {
      const extractRes = extractSheetIds(chat.webSummarySheetLink)
      if (!extractRes) {
        logger.error("Failed to extract sheetToken and range")
        return ""
      }
      const wikiData = await larkService.wiki.getNodeInfo(extractRes.sheetToken)
      if (!wikiData || wikiData.code) {
        logger.error("Failed to get wiki data")
        return ""
      }
      sheetToken = wikiData.data.obj_token
      range = extractRes.range
    } else {
      const extractRes = extractSheetIds(chat.webSummarySheetLink)
      if (!extractRes) {
        logger.error("Failed to extract sheetToken and range")
        return ""
      }
      sheetToken = extractRes.sheetToken
      range = extractRes.range
    }

    await larkService.sheet.insertRows(sheetToken, range, [
      [
        userId || "",
        link,
        userDescription,
        llmRes,
        new Date().toLocaleString("zh-CN", { timeZone: "Asia/Shanghai" }),
      ],
    ])
    return chat.webSummarySheetLink
  } catch (error: any) {
    logger.error(`Failed to insert2Sheet: ${error}`)
    return ""
  }
}

/**
 * 生成简报卡片
 * @param {Context} ctx - 上下文对象
 * @param {string} link - 网页链接
 * @param {string} userDescription - 用户描述
 * @param {string} llmRes - 简报内容
 * @param {string} sheetLink - 表格链接
 * @returns {any} - 返回卡片对象
 */
const genReportCard = (
  ctx: Context,
  link: string,
  userDescription: string,
  llmRes: string,
  sheetLink: string
) => {
  const { larkCard } = ctx
  const cardGender = larkCard.child("webAgent")
  const description = userDescription
    ? `**用户描述📝**\n${userDescription}\n`
    : ""
  const sheetLinkMd = sheetLink
    ? `**小提示🔍**
您可以在[这里](${sheetLink})查看往期简报哦！`
    : ""
  const content = `
感谢您使用小煎蛋简报🍳！以下是为您总结的简报内容：

**网站地址🌟**
[${link}](${link})

${description}

**AI简报🌈**
${llmRes}

${sheetLinkMd}
`
  return cardGender.genCard("markdownSuccessCard", {
    content,
  })
}

/**
 * 生成简报
 * @param {Context} ctx - 上下文对象
 * @param {string} link - 网页链接
 * @param {string} userDescription - 用户描述
 */
const agent = async (ctx: Context, link: string, userDescription: string) => {
  const {
    larkService: { message },
    larkCard,
    logger,
  } = ctx
  const cardGender = larkCard.child("webAgent")
  try {
    // 校验链接是否合法
    validateLink(link)
    // 发送一个loading卡片
    await message.updateOrReply(
      cardGender.genSuccessCard("正在为您收集简报，请稍等片刻~")
    )
    // 抓取网页
    const crawRes = await crawlWebPage(ctx, link)
    // 调用模型生成简报
    const llmRes = await generateSummary(ctx, userDescription, crawRes)
    // 插入到表格
    const sheetLink = await insert2Sheet(ctx, link, userDescription, llmRes)
    // 发送简报卡片
    await message.updateOrReply(
      genReportCard(ctx, link, userDescription, llmRes, sheetLink)
    )
  } catch (error: any) {
    logger.error(`Failed gen report: ${error}`)
    await message.updateOrReply(
      cardGender.genErrorCard(`简报生成失败: ${error.message}`)
    )
  }
}

const reportAgent = {
  agent,
}

export default reportAgent
